﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;

namespace PicoScope.AutomationDemo.Automation
{
  /// <summary>
  /// Runs automation commands through the command line interface
  /// </summary>
  public class ProcessAutomation
  {
    private static readonly Lazy<ProcessAutomation> Lazy = new Lazy<ProcessAutomation>(() => new ProcessAutomation());
    private string _lastKnownPicoPath;

    private ProcessAutomation()
    {
    }

    public static ProcessAutomation Instance
    {
      get { return Lazy.Value; }
    }

    public string RunCommand(string cmd)
    {
      CheckPicoFound();

      try
      {
        return ExecuteProcess(cmd);
      }
      catch (PicoScopeNowFoundException)
      {
        _lastKnownPicoPath = null;
        CheckPicoFound();
      }

      return ExecuteProcess(cmd);
    }

    private string ExecuteProcess(string cmd)
    {
      ProcessStartInfo info = new ProcessStartInfo(_lastKnownPicoPath, "/a " + cmd)
        {
          UseShellExecute = false,
          RedirectStandardOutput = true
        };

      Process process = Process.Start(info);
      process.WaitForExit();

      string output = process.StandardOutput.ReadToEnd();

      if(output.ToLower().Contains("could not find an instance"))
        throw new PicoScopeNowFoundException();

      return output;
    }

    private void CheckPicoFound()
    {
      if (!string.IsNullOrEmpty(_lastKnownPicoPath))
        return;

      _lastKnownPicoPath = GetBestPicoPath();

      if (string.IsNullOrEmpty(_lastKnownPicoPath))
        throw new PicoScopeNowFoundException();
    }

    private string GetBestPicoPath()
    {
      var topProcess = GetPicoProcesses()
        .Select(x => new
          {
            Path = x.MainModule.FileName,
            Version = FileVersionInfo.GetVersionInfo(x.MainModule.FileName)
          })
        .OrderBy(ob => ob.Version)
        .FirstOrDefault();

      if (topProcess == null)
        return null;

      return Path.Combine(new FileInfo(topProcess.Path).DirectoryName, "PicoScope.exe");
    }

    private IEnumerable<Process> GetPicoProcesses()
    {
      foreach (var each in Process.GetProcessesByName("PicoScope.vshost"))
        yield return each;

      foreach (var each in Process.GetProcessesByName("PicoScope"))
        yield return each;
    }
  }
}