﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mime;
using System.Reflection;
using System.Text;
using Griffin.Networking.Buffers;
using Griffin.Networking.Protocol.Http;
using Griffin.Networking.Protocol.Http.Implementation;
using Griffin.Networking.Protocol.Http.Protocol;
using PicoScope.AutomationDemo.Automation;

namespace PicoScope.AutomationDemo
{
  public class PicoHttpServer : HttpService
  {
    private static readonly BufferSliceStack Stack = new BufferSliceStack(50, 32000);

    /// <summary>
    /// Initializes a new instance of the <see cref="PicoHttpServer"/> class.
    /// </summary>
    /// <param name="basePath">The base path.</param>
    public PicoHttpServer()
      : base(Stack)
    {
    }

    /// <summary>
    /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
    /// </summary>
    public override void Dispose()
    {
    }

    /// <summary>
    /// A new message have been received from the remote end.
    /// </summary>
    /// <param name="message">You'll receive <see cref="T:Griffin.Networking.Protocol.Http.Protocol.IRequest" /> or <see cref="T:Griffin.Networking.Protocol.Http.Protocol.IResponse" /> depending on the type of application.</param>
    public override void HandleReceive(object message)
    {
      List<Func<HttpRequest, bool>> processors = new List<Func<HttpRequest, bool>>
        {
          AutomationProxy,
          NotFound
        };

      HttpRequest request = message as HttpRequest;

      try
      {
        foreach (var processor in processors)
          if (processor(request))
            return;
      }
      catch (Exception ex)
      {
        Error(request, ex);
      }
    }


    /// <summary>
    /// Automations comms with the automoation proxy.
    /// </summary>
    /// <param name="request">The request.</param>
    /// <returns></returns>
    private bool AutomationProxy(HttpRequest request)
    {
      if (!request.Uri.AbsolutePath.ToLower().StartsWith("/automation"))
        return false;

      using (StreamReader incoming = new StreamReader(request.Body, Encoding.UTF8))
      {
        try
        {
          string responseString = IpcAutomation.Instance.RunCommand(incoming.ReadToEnd());

          IResponse response = request.CreateResponse(HttpStatusCode.OK, "OK");
          response.Body = WriteStringToBody(responseString);
          response.ContentType = "txt";
          Send(response);
        }
        catch (PicoScopeNowFoundException)
        {
          IResponse response = request.CreateResponse(HttpStatusCode.ServiceUnavailable, "PicoScope Not Found");
          Send(response);
        }
      }

      return true;
    }

    /// <summary>
    /// Handles nots found.
    /// </summary>
    /// <param name="request">The request.</param>
    /// <returns></returns>
    private bool NotFound(HttpRequest request)
    {
      IResponse response = request.CreateResponse(HttpStatusCode.NotFound, "Not Found");
      Send(response);
      return true;
    }

    /// <summary>
    /// Writes the string to body.
    /// </summary>
    /// <param name="input">The input.</param>
    /// <returns></returns>
    private static Stream WriteStringToBody(string input)
    {
      byte[] buffer = Encoding.UTF8.GetBytes(input);
      MemoryStream body = new MemoryStream();
      body.Write(buffer, 0, buffer.Length);
      body.Position = 0;
      return body;
    }

    /// <summary>
    /// Returns an error.
    /// </summary>
    /// <param name="request">The request.</param>
    /// <param name="ex">The exception</param>
    private void Error(IRequest request, Exception ex)
    {
      IResponse response = request.CreateResponse(HttpStatusCode.InternalServerError, ex.GetType().Name);
      response.Body = WriteStringToBody(ex.ToString());
      response.ContentType = "text/plain";
      Send(response);
    }
  }
}